@@ -299,9 +299,10 @@ mod tests {
     use std::collections::VecDeque;
     use std::sync::{Arc, Mutex};
 
+    use cylinder::{PublicKey, Signature, VerificationError, Verifier};
     use protobuf::Message;
 
-    use crate::network::auth::create_authorization_dispatcher;
+    use crate::network::auth::AuthorizationDispatchBuilder;
     use crate::protos::authorization;
     use crate::protos::network::{NetworkMessage, NetworkMessageType};
 
@@ -317,13 +318,24 @@ mod tests {
         let auth_mgr = AuthorizationManagerStateMachine::default();
         let mock_sender = MockSender::new();
         let dispatch_sender = mock_sender.clone();
-        let dispatcher = create_authorization_dispatcher(
-            "mock_identity".into(),
-            #[cfg(feature = "challenge-authorization")]
-            vec![],
-            auth_mgr,
-            dispatch_sender,
-        );
+        // mut is required if chalenge authorization is enabled
+        #[allow(unused_mut)]
+        let mut dispatcher_builder =
+            AuthorizationDispatchBuilder::new().with_identity("mock_identity");
+
+        #[cfg(feature = "challenge-authorization")]
+        {
+            dispatcher_builder = dispatcher_builder
+                .with_signers(&vec![])
+                .with_nonce(&vec![])
+                .with_expected_authorization(None)
+                .with_local_authorization(None)
+                .with_verifier(Box::new(NoopVerifier))
+        }
+
+        let dispatcher = dispatcher_builder
+            .build(dispatch_sender, auth_mgr)
+            .expect("Unable to build authorization dispatcher");
 
         let connection_id = "test_connection".to_string();
         let mut msg = authorization::ConnectRequest::new();
@@ -381,13 +393,24 @@ mod tests {
         let auth_mgr = AuthorizationManagerStateMachine::default();
         let mock_sender = MockSender::new();
         let dispatch_sender = mock_sender.clone();
-        let dispatcher = create_authorization_dispatcher(
-            "mock_identity".into(),
-            #[cfg(feature = "challenge-authorization")]
-            vec![],
-            auth_mgr,
-            dispatch_sender,
-        );
+        // mut is required if chalenge authorization is enabled
+        #[allow(unused_mut)]
+        let mut dispatcher_builder =
+            AuthorizationDispatchBuilder::new().with_identity("mock_identity");
+
+        #[cfg(feature = "challenge-authorization")]
+        {
+            dispatcher_builder = dispatcher_builder
+                .with_signers(&vec![])
+                .with_nonce(&vec![])
+                .with_expected_authorization(None)
+                .with_local_authorization(None)
+                .with_verifier(Box::new(NoopVerifier))
+        }
+
+        let dispatcher = dispatcher_builder
+            .build(dispatch_sender, auth_mgr)
+            .expect("Unable to build authorization dispatcher");
         let connection_id = "test_connection".to_string();
         let mut msg = authorization::ConnectResponse::new();
         msg.set_accepted_authorization_types(
@@ -430,13 +453,24 @@ mod tests {
         let auth_mgr = AuthorizationManagerStateMachine::default();
         let mock_sender = MockSender::new();
         let dispatch_sender = mock_sender.clone();
-        let dispatcher = create_authorization_dispatcher(
-            "mock_identity".into(),
-            #[cfg(feature = "challenge-authorization")]
-            vec![],
-            auth_mgr,
-            dispatch_sender,
-        );
+        // mut is required if chalenge authorization is enabled
+        #[allow(unused_mut)]
+        let mut dispatcher_builder =
+            AuthorizationDispatchBuilder::new().with_identity("mock_identity");
+
+        #[cfg(feature = "challenge-authorization")]
+        {
+            dispatcher_builder = dispatcher_builder
+                .with_signers(&vec![])
+                .with_nonce(&vec![])
+                .with_expected_authorization(None)
+                .with_local_authorization(None)
+                .with_verifier(Box::new(NoopVerifier))
+        }
+
+        let dispatcher = dispatcher_builder
+            .build(dispatch_sender, auth_mgr)
+            .expect("Unable to build authorization dispatcher");
         let connection_id = "test_connection".to_string();
         // Begin the connection process, otherwise, the response will fail
         let mut msg = authorization::ConnectRequest::new();
@@ -539,4 +573,21 @@ mod tests {
             Ok(())
         }
     }
+
+    struct NoopVerifier;
+
+    impl Verifier for NoopVerifier {
+        fn algorithm_name(&self) -> &str {
+            unimplemented!()
+        }
+
+        fn verify(
+            &self,
+            _message: &[u8],
+            _signature: &Signature,
+            _public_key: &PublicKey,
+        ) -> Result<bool, VerificationError> {
+            unimplemented!()
+        }
+    }
 }
