@@ -12,8 +12,11 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+#[cfg(feature = "diesel")]
+pub mod diesel;
 /// NodeIdStore error types
 pub mod error;
+
 use error::NodeIdStoreError;
 
 /// Trait for interacting with the instances node_id.
