@@ -123,7 +123,11 @@ pub fn make_callback_route(
                                         ) {
                                             Ok(_) => debug!("User profile saved"),
                                             Err(err) => {
-                                                error!("Failed to save profile: {}", err);
+                                                error!(
+                                                    "Failed to save profile for account: {}, {}",
+                                                    user_info.subject.clone(),
+                                                    err
+                                                );
                                                 return Box::new(
                                                     HttpResponse::InternalServerError()
                                                         .json(ErrorResponse::internal_error())
@@ -229,7 +233,11 @@ pub fn make_callback_route(
                                     ) {
                                         Ok(_) => debug!("User profile saved"),
                                         Err(err) => {
-                                            error!("Failed to save profile: {}", err);
+                                            error!(
+                                                "Failed to save profile for account: {}, {}",
+                                                user_info.subject.clone(),
+                                                err
+                                            );
                                             return Box::new(
                                                 HttpResponse::InternalServerError()
                                                     .json(ErrorResponse::internal_error())
@@ -342,7 +350,6 @@ mod tests {
         PendingAuthorization,
     };
 
-    #[cfg(feature = "oauth-profile")]
     use crate::oauth::tests::TestProfileProvider;
 
     const TOKEN_ENDPOINT: &str = "/token";
@@ -398,7 +405,6 @@ mod tests {
             vec![],
             Box::new(TestSubjectProvider),
             request_store.clone(),
-            #[cfg(feature = "oauth-profile")]
             Box::new(TestProfileProvider),
         );
 
@@ -507,7 +513,6 @@ mod tests {
             vec![],
             Box::new(TestSubjectProvider),
             Box::new(MemoryInflightOAuthRequestStore::new()),
-            #[cfg(feature = "oauth-profile")]
             Box::new(TestProfileProvider),
         );
 
@@ -588,7 +593,6 @@ mod tests {
             vec![],
             Box::new(TestSubjectProvider),
             request_store.clone(),
-            #[cfg(feature = "oauth-profile")]
             Box::new(TestProfileProvider),
         );
 
@@ -671,7 +675,6 @@ mod tests {
             vec![],
             Box::new(TestSubjectProvider),
             request_store.clone(),
-            #[cfg(feature = "oauth-profile")]
             Box::new(TestProfileProvider),
         );
 
