@@ -1,4 +1,4 @@
-// Copyright 2018-2020 Cargill Incorporated
+// Copyright 2018-2021 Cargill Incorporated
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -18,32 +18,59 @@
 //! * `PUT /registry/nodes/{identity}` for replacing a node in the registry
 //! * `DELETE /registry/nodes/{identity}` for deleting a node from the registry
 
+use std::convert::TryFrom;
+
 use crate::actix_web::{error::BlockingError, web, Error, HttpRequest, HttpResponse};
+use crate::error::InvalidStateError;
 use crate::futures::{future::IntoFuture, stream::Stream, Future};
 use crate::protocol;
+use crate::registry::rest_api::error::RegistryRestApiError;
+#[cfg(feature = "authorization")]
+use crate::registry::rest_api::{REGISTRY_READ_PERMISSION, REGISTRY_WRITE_PERMISSION};
 use crate::registry::{
-    rest_api::resources::nodes_identity::NodeResponse, InvalidNodeError, Node, RegistryError,
-    RegistryReader, RegistryWriter, RwRegistry,
+    rest_api::resources::nodes_identity::{NewNode, NodeResponse},
+    Node, RegistryReader, RegistryWriter, RwRegistry,
+};
+use crate::rest_api::{
+    actix_web_1::{Method, ProtocolVersionRangeGuard, Resource},
+    ErrorResponse,
 };
-use crate::rest_api::{ErrorResponse, Method, ProtocolVersionRangeGuard, Resource};
 
 pub fn make_nodes_identity_resource(registry: Box<dyn RwRegistry>) -> Resource {
     let registry1 = registry.clone();
     let registry2 = registry.clone();
-    Resource::build("/registry/nodes/{identity}")
-        .add_request_guard(ProtocolVersionRangeGuard::new(
+    let resource = Resource::build("/registry/nodes/{identity}").add_request_guard(
+        ProtocolVersionRangeGuard::new(
             protocol::REGISTRY_FETCH_NODE_MIN,
             protocol::REGISTRY_PROTOCOL_VERSION,
-        ))
-        .add_method(Method::Get, move |r, _| {
-            fetch_node(r, web::Data::new(registry.clone_box_as_reader()))
-        })
-        .add_method(Method::Put, move |r, p| {
-            put_node(r, p, web::Data::new(registry1.clone_box_as_writer()))
-        })
-        .add_method(Method::Delete, move |r, _| {
-            delete_node(r, web::Data::new(registry2.clone_box_as_writer()))
-        })
+        ),
+    );
+    #[cfg(feature = "authorization")]
+    {
+        resource
+            .add_method(Method::Get, REGISTRY_READ_PERMISSION, move |r, _| {
+                fetch_node(r, web::Data::new(registry.clone_box_as_reader()))
+            })
+            .add_method(Method::Put, REGISTRY_WRITE_PERMISSION, move |r, p| {
+                put_node(r, p, web::Data::new(registry1.clone_box_as_writer()))
+            })
+            .add_method(Method::Delete, REGISTRY_WRITE_PERMISSION, move |r, _| {
+                delete_node(r, web::Data::new(registry2.clone_box_as_writer()))
+            })
+    }
+    #[cfg(not(feature = "authorization"))]
+    {
+        resource
+            .add_method(Method::Get, move |r, _| {
+                fetch_node(r, web::Data::new(registry.clone_box_as_reader()))
+            })
+            .add_method(Method::Put, move |r, p| {
+                put_node(r, p, web::Data::new(registry1.clone_box_as_writer()))
+            })
+            .add_method(Method::Delete, move |r, _| {
+                delete_node(r, web::Data::new(registry2.clone_box_as_writer()))
+            })
+    }
 }
 
 fn fetch_node(
@@ -56,7 +83,12 @@ fn fetch_node(
         .unwrap_or("")
         .to_string();
     Box::new(
-        web::block(move || registry.fetch_node(&identity)).then(|res| {
+        web::block(move || {
+            registry
+                .get_node(&identity)
+                .map_err(RegistryRestApiError::from)
+        })
+        .then(|res| {
             Ok(match res {
                 Ok(Some(node)) => HttpResponse::Ok().json(NodeResponse::from(&node)),
                 Ok(None) => {
@@ -89,28 +121,39 @@ fn put_node(
                 Ok::<_, Error>(body)
             })
             .into_future()
-            .and_then(move |body| match serde_json::from_slice::<Node>(&body) {
+            .and_then(move |body| match serde_json::from_slice::<NewNode>(&body) {
                 Ok(node) => Box::new(
                     web::block(move || {
-                        if node.identity != path_identity {
-                            Err(RegistryError::InvalidNode(
-                                InvalidNodeError::InvalidIdentity(
-                                    node.identity,
-                                    "Node identity cannot be changed".into(),
-                                ),
+                        let update_node = Node::try_from(node).map_err(|err| {
+                            RegistryRestApiError::InvalidStateError(
+                                InvalidStateError::with_message(format!(
+                                    "Failed to update node, node is invalid: {}",
+                                    err
+                                )),
+                            )
+                        })?;
+
+                        if update_node.identity != path_identity {
+                            Err(RegistryRestApiError::InvalidStateError(
+                                InvalidStateError::with_message(format!(
+                                    "Node identity cannot be changed: {}",
+                                    update_node.identity
+                                )),
                             ))
                         } else {
-                            registry.insert_node(node)
+                            registry
+                                .update_node(update_node)
+                                .map_err(RegistryRestApiError::from)
                         }
                     })
                     .then(|res| {
                         Ok(match res {
                             Ok(_) => HttpResponse::Ok().finish(),
-                            Err(BlockingError::Error(RegistryError::InvalidNode(err))) => {
-                                HttpResponse::BadRequest().json(ErrorResponse::bad_request(
-                                    &format!("Invalid node: {}", err),
-                                ))
-                            }
+                            Err(BlockingError::Error(RegistryRestApiError::InvalidStateError(
+                                err,
+                            ))) => HttpResponse::BadRequest().json(ErrorResponse::bad_request(
+                                &format!("Invalid node: {}", err),
+                            )),
                             Err(err) => {
                                 error!("Unable to put node: {}", err);
                                 HttpResponse::InternalServerError()
@@ -142,7 +185,12 @@ fn delete_node(
         .unwrap_or("")
         .to_string();
     Box::new(
-        web::block(move || registry.delete_node(&identity)).then(|res| {
+        web::block(move || {
+            registry
+                .delete_node(&identity)
+                .map_err(RegistryRestApiError::from)
+        })
+        .then(|res| {
             Ok(match res {
                 Ok(Some(_)) => HttpResponse::Ok().finish(),
                 Ok(None) => {
@@ -166,8 +214,9 @@ mod tests {
 
     use reqwest::{blocking::Client, StatusCode, Url};
 
-    use crate::registry::{MetadataPredicate, NodeIter};
-    use crate::rest_api::{RestApiBuilder, RestApiServerError, RestApiShutdownHandle};
+    use crate::error::InvalidStateError;
+    use crate::registry::{error::RegistryError, MetadataPredicate, NodeIter};
+    use crate::rest_api::actix_web_1::{RestApiBuilder, RestApiShutdownHandle};
 
     #[test]
     /// Tests a GET /registry/nodes/{identity} request returns the expected node.
@@ -193,8 +242,11 @@ mod tests {
             .expect("Failed to perform request");
 
         assert_eq!(resp.status(), StatusCode::OK);
-        let node: Node = resp.json().expect("Failed to deserialize body");
-        assert_eq!(node, get_node_1());
+        let node: TestNode = resp.json().expect("Failed to deserialize body");
+        assert_eq!(
+            Node::try_from(node).expect("Unable to build node"),
+            get_node_1()
+        );
 
         shutdown_handle
             .shutdown()
@@ -261,7 +313,7 @@ mod tests {
 
         // Verify that updating an existing node gets an OK response and the fetched node has
         // the updated values
-        let mut node = get_node_1();
+        let mut node = get_new_node_1();
         node.endpoints = vec!["12.0.0.123:8432".to_string()];
         node.metadata
             .insert("location".to_string(), "Minneapolis".to_string());
@@ -293,7 +345,7 @@ mod tests {
             .expect("Failed to perform request");
 
         assert_eq!(resp.status(), StatusCode::OK);
-        let updated_node: Node = resp.json().expect("Failed to deserialize body");
+        let updated_node: NewNode = resp.json().expect("Failed to deserialize body");
         assert_eq!(updated_node, node);
 
         // Verify that attempting to change the node identity gets a FORBIDDEN response
@@ -370,24 +422,24 @@ mod tests {
     fn run_rest_api_on_open_port(
         resources: Vec<Resource>,
     ) -> (RestApiShutdownHandle, std::thread::JoinHandle<()>, String) {
-        (10000..20000)
-            .find_map(|port| {
-                let bind_url = format!("127.0.0.1:{}", port);
-                let result = RestApiBuilder::new()
-                    .with_bind(&bind_url)
-                    .add_resources(resources.clone())
-                    .build()
-                    .expect("Failed to build REST API")
-                    .run();
-                match result {
-                    Ok((shutdown_handle, join_handle)) => {
-                        Some((shutdown_handle, join_handle, bind_url))
-                    }
-                    Err(RestApiServerError::BindError(_)) => None,
-                    Err(err) => panic!("Failed to run REST API: {}", err),
-                }
-            })
-            .expect("No port available")
+        #[cfg(not(feature = "https-bind"))]
+        let bind = "127.0.0.1:0";
+        #[cfg(feature = "https-bind")]
+        let bind = crate::rest_api::BindConfig::Http("127.0.0.1:0".into());
+
+        let result = RestApiBuilder::new()
+            .with_bind(bind)
+            .add_resources(resources.clone())
+            .build_insecure()
+            .expect("Failed to build REST API")
+            .run_insecure();
+        match result {
+            Ok((shutdown_handle, join_handle)) => {
+                let port = shutdown_handle.port_numbers()[0];
+                (shutdown_handle, join_handle, format!("127.0.0.1:{}", port))
+            }
+            Err(err) => panic!("Failed to run REST API: {}", err),
+        }
     }
 
     fn get_node_1() -> Node {
@@ -400,6 +452,19 @@ mod tests {
             .expect("Failed to build node1")
     }
 
+    fn get_new_node_1() -> NewNode {
+        let mut metadata = HashMap::new();
+        metadata.insert("company".into(), "Bitwise IO".into());
+
+        NewNode {
+            identity: "Node-123".into(),
+            endpoints: vec!["12.0.0.123:8431".into()],
+            display_name: "Bitwise IO - Node 1".into(),
+            keys: vec!["0123".into()],
+            metadata,
+        }
+    }
+
     fn get_node_2() -> Node {
         Node::builder("Node-456")
             .with_endpoint("13.0.0.123:8434")
@@ -410,6 +475,33 @@ mod tests {
             .expect("Failed to build node2")
     }
 
+    /// Test representation of a node in a registry.
+    #[derive(Debug, Serialize, Deserialize, Clone, PartialEq)]
+    struct TestNode {
+        identity: String,
+        endpoints: Vec<String>,
+        display_name: String,
+        keys: Vec<String>,
+        metadata: HashMap<String, String>,
+    }
+
+    impl TryFrom<TestNode> for Node {
+        type Error = String;
+
+        fn try_from(node: TestNode) -> Result<Self, Self::Error> {
+            let mut builder = Node::builder(node.identity)
+                .with_endpoints(node.endpoints)
+                .with_display_name(node.display_name)
+                .with_keys(node.keys);
+
+            for (k, v) in node.metadata {
+                builder = builder.with_metadata(k, v);
+            }
+
+            builder.build().map_err(|err| err.to_string())
+        }
+    }
+
     #[derive(Clone, Default)]
     struct MemRegistry {
         nodes: Arc<Mutex<HashMap<String, Node>>>,
@@ -445,7 +537,7 @@ mod tests {
             self.list_nodes(predicates).map(|iter| iter.count() as u32)
         }
 
-        fn fetch_node(&self, identity: &str) -> Result<Option<Node>, RegistryError> {
+        fn get_node(&self, identity: &str) -> Result<Option<Node>, RegistryError> {
             Ok(self
                 .nodes
                 .lock()
@@ -456,7 +548,7 @@ mod tests {
     }
 
     impl RegistryWriter for MemRegistry {
-        fn insert_node(&self, node: Node) -> Result<(), RegistryError> {
+        fn add_node(&self, node: Node) -> Result<(), RegistryError> {
             self.nodes
                 .lock()
                 .expect("mem registry lock was poisoned")
@@ -464,6 +556,22 @@ mod tests {
             Ok(())
         }
 
+        fn update_node(&self, node: Node) -> Result<(), RegistryError> {
+            let mut inner = self.nodes.lock().expect("mem registry lock was poisoned");
+
+            if inner.contains_key(&node.identity) {
+                inner.insert(node.identity.clone(), node);
+                Ok(())
+            } else {
+                Err(RegistryError::InvalidStateError(
+                    InvalidStateError::with_message(format!(
+                        "Node does not exist in the registry: {}",
+                        node.identity
+                    )),
+                ))
+            }
+        }
+
         fn delete_node(&self, identity: &str) -> Result<Option<Node>, RegistryError> {
             Ok(self
                 .nodes
