@@ -1,4 +1,4 @@
-// Copyright 2018-2020 Cargill Incorporated
+// Copyright 2018-2021 Cargill Incorporated
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -20,6 +20,7 @@
 //! [`RemoteYamlRegistry`]: struct.RemoteYamlRegistry.html
 //! [`RegistryReader`]: ../../trait.RegistryReader.html
 
+use std::convert::TryFrom;
 use std::path::Path;
 use std::sync::{
     atomic::{AtomicBool, Ordering},
@@ -30,12 +31,15 @@ use std::time::{Duration, Instant};
 
 use openssl::hash::{hash, MessageDigest};
 
+use crate::error::{InternalError, InvalidStateError};
 use crate::hex::to_hex;
 use crate::registry::{
-    validate_nodes, MetadataPredicate, Node, NodeIter, RegistryError, RegistryReader,
+    error::InvalidNodeError, validate_nodes, MetadataPredicate, Node, NodeIter, RegistryError,
+    RegistryReader,
 };
+use crate::threading::lifecycle::ShutdownHandle;
 
-use super::LocalYamlRegistry;
+use super::{LocalYamlRegistry, YamlNode};
 
 /// A remote, read-only registry.
 ///
@@ -72,7 +76,7 @@ use super::LocalYamlRegistry;
 /// [`constructor`]: struct.RemoteYamlRegistry.html#method.new
 pub struct RemoteYamlRegistry {
     internal: Arc<Mutex<Internal>>,
-    shutdown_handle: ShutdownHandle,
+    shutdown_handle: Option<RemoteYamlShutdownHandle>,
 }
 
 impl RemoteYamlRegistry {
@@ -100,14 +104,14 @@ impl RemoteYamlRegistry {
             forced_refresh_period,
         )?));
 
-        let running = automatic_refresh_period
-            .map::<Result<_, RegistryError>, _>(|refresh_period| {
+        let (running, join_handle) = {
+            if let Some(refresh_period) = automatic_refresh_period {
                 let running = Arc::new(AtomicBool::new(true));
 
                 let thread_internal = internal.clone();
                 let thread_url = url.to_string();
                 let thread_running = running.clone();
-                thread::Builder::new()
+                let join_handle = thread::Builder::new()
                     .name(format!("Remote Registry Automatic Refresh: {}", url))
                     .spawn(move || {
                         automatic_refresh_loop(
@@ -118,41 +122,50 @@ impl RemoteYamlRegistry {
                         )
                     })
                     .map_err(|err| {
-                        RegistryError::general_error_with_source(
-                            &format!(
+                        RegistryError::InternalError(InternalError::from_source_with_message(
+                            Box::new(err),
+                            format!(
                                 "Failed to spawn automatic refresh thread for remote registry '{}'",
                                 url
                             ),
-                            Box::new(err),
-                        )
+                        ))
                     })?;
-                Ok(running)
-            })
-            .transpose()?;
-        let shutdown_handle = ShutdownHandle { running };
+                (Some(running), Some(join_handle))
+            } else {
+                (None, None)
+            }
+        };
+
+        let shutdown_handle = RemoteYamlShutdownHandle {
+            running,
+            join_handle,
+        };
 
         Ok(Self {
             internal,
-            shutdown_handle,
+            shutdown_handle: Some(shutdown_handle),
         })
     }
 
-    /// Get a copy of the registry's `ShutdownHandle`.
-    pub fn shutdown_handle(&self) -> ShutdownHandle {
-        self.shutdown_handle.clone()
+    pub fn take_shutdown_handle(&mut self) -> Option<RemoteYamlShutdownHandle> {
+        self.shutdown_handle.take()
     }
 
     /// Acquire the lock for the internal cache and get the nodes from it.
     fn get_nodes(&self) -> Result<Vec<Node>, RegistryError> {
         self.internal
             .lock()
-            .map_err(|_| RegistryError::general_error("Internal lock poisoned"))?
+            .map_err(|_| {
+                RegistryError::InternalError(InternalError::with_message(
+                    "Internal lock poisoned".into(),
+                ))
+            })?
             .get_nodes()
     }
 }
 
 impl RegistryReader for RemoteYamlRegistry {
-    fn fetch_node(&self, identity: &str) -> Result<Option<Node>, RegistryError> {
+    fn get_node(&self, identity: &str) -> Result<Option<Node>, RegistryError> {
         Ok(self
             .get_nodes()?
             .into_iter()
@@ -233,10 +246,11 @@ impl Internal {
                     .forced_refresh_period
                     .map(|duration| {
                         Instant::now().checked_add(duration).ok_or_else(|| {
-                            RegistryError::general_error(
+                            RegistryError::InternalError(InternalError::with_message(
                                 "Forced refresh time could not be determined; \
-                                 forced_refresh_period may be too large",
-                            )
+                                 forced_refresh_period may be too large"
+                                    .into(),
+                            ))
                         })
                     })
                     .transpose()?;
@@ -284,11 +298,12 @@ fn compute_cache_filename(url: &str, cache_dir: &str) -> Result<String, Registry
     let hash = hash(MessageDigest::sha256(), url.as_bytes())
         .map(|digest| to_hex(&*digest))
         .map_err(|err| {
-            RegistryError::general_error_with_source(
-                "Failed to hash URL for cache file",
+            RegistryError::InternalError(InternalError::from_source_with_message(
                 Box::new(err),
-            )
+                "Failed to hash URL for cache file".into(),
+            ))
         })?;
+
     let filename = format!("remote_registry_{}.yaml", hash);
     Ok(Path::new(cache_dir)
         .join(filename)
@@ -302,25 +317,38 @@ fn fetch_nodes_from_remote(url: &str) -> Result<Vec<Node>, RegistryError> {
     let bytes = reqwest::blocking::get(url)
         .and_then(|response| response.error_for_status())
         .map_err(|err| {
-            RegistryError::general_error_with_source(
-                &format!("Failed to fetch remote registry file from {}", url),
+            RegistryError::InternalError(InternalError::from_source_with_message(
                 Box::new(err),
-            )
+                format!("Failed to fetch remote registry file from {}", url),
+            ))
         })?
         .bytes()
         .map_err(|err| {
-            RegistryError::general_error_with_source(
-                "Failed to get bytes from remote registry file HTTP response",
+            RegistryError::InternalError(InternalError::from_source_with_message(
                 Box::new(err),
-            )
+                "Failed to get bytes from remote registry file HTTP response".into(),
+            ))
         })?;
-    let nodes: Vec<Node> = serde_yaml::from_slice(&bytes).map_err(|_| {
-        RegistryError::general_error(
-            "Failed to deserialize remote registry file: Not a valid YAML sequence of nodes",
-        )
+    let yaml_nodes: Vec<YamlNode> = serde_yaml::from_slice(&bytes).map_err(|_| {
+        RegistryError::InternalError(InternalError::with_message(
+            "Failed to deserialize remote registry file: Not a valid YAML sequence of nodes".into(),
+        ))
     })?;
 
-    validate_nodes(&nodes)?;
+    let nodes: Vec<Node> = yaml_nodes
+        .into_iter()
+        .map(Node::try_from)
+        .collect::<Result<Vec<Node>, InvalidNodeError>>()
+        .map_err(|err| {
+            RegistryError::InvalidStateError(InvalidStateError::with_message(format!(
+                "Unable to get node list: {}",
+                err
+            )))
+        })?;
+
+    validate_nodes(&nodes).map_err(|err| {
+        RegistryError::InvalidStateError(InvalidStateError::with_message(err.to_string()))
+    })?;
 
     Ok(nodes)
 }
@@ -376,18 +404,29 @@ fn automatic_refresh_loop(
 }
 
 /// Handle for signaling the `RemoteYamlRegistry` to shutdown.
-#[derive(Clone)]
-pub struct ShutdownHandle {
+pub struct RemoteYamlShutdownHandle {
     running: Option<Arc<AtomicBool>>,
+    join_handle: Option<thread::JoinHandle<()>>,
 }
 
-impl ShutdownHandle {
+impl ShutdownHandle for RemoteYamlShutdownHandle {
     /// Send shutdown signal to `RemoteYamlRegistry`.
-    pub fn shutdown(&self) {
+    fn signal_shutdown(&mut self) {
         if let Some(running) = &self.running {
             running.store(false, Ordering::SeqCst)
         }
     }
+
+    fn wait_for_shutdown(self) -> Result<(), InternalError> {
+        if let Some(join_handle) = self.join_handle {
+            if join_handle.join().is_err() {
+                return Err(InternalError::with_message(
+                    "Unable to shutdown remote yaml registry".to_string(),
+                ));
+            }
+        }
+        Ok(())
+    }
 }
 
 #[cfg(all(test, feature = "rest-api", feature = "rest-api-actix"))]
@@ -400,9 +439,9 @@ mod tests {
     use futures::future::IntoFuture;
     use tempdir::TempDir;
 
-    use crate::rest_api::{
-        Method, Resource, RestApiBuilder, RestApiServerError, RestApiShutdownHandle,
-    };
+    use crate::rest_api::actix_web_1::{Method, Resource, RestApiBuilder, RestApiShutdownHandle};
+    #[cfg(feature = "authorization")]
+    use crate::rest_api::auth::authorization::Permission;
 
     /// Verifies that a remote file that contains two nodes with the same identity is rejected (not
     /// loaded).
@@ -413,14 +452,20 @@ mod tests {
         registry[1].identity = "identity".into();
         let test_config = TestConfig::setup("duplicate_identity", Some(registry));
 
-        let remote_registry =
+        let mut remote_registry =
             RemoteYamlRegistry::new(test_config.url(), test_config.path(), None, None)
                 .expect("Failed to create registry");
 
         // Verify that the registry is still empty
         verify_internal_cache(&test_config, &remote_registry, vec![]);
 
-        remote_registry.shutdown_handle().shutdown();
+        let mut shutdown_handle = remote_registry
+            .take_shutdown_handle()
+            .expect("Unable to get shutdown handle");
+        shutdown_handle.signal_shutdown();
+        shutdown_handle
+            .wait_for_shutdown()
+            .expect("Unable to shutdown remote registry");
         test_config.shutdown();
     }
 
@@ -433,14 +478,20 @@ mod tests {
         registry[1].endpoints = vec!["endpoint".into()];
         let test_config = TestConfig::setup("duplicate_endpoint", Some(registry));
 
-        let remote_registry =
+        let mut remote_registry =
             RemoteYamlRegistry::new(test_config.url(), test_config.path(), None, None)
                 .expect("Failed to create registry");
 
         // Verify that the registry is still empty
         verify_internal_cache(&test_config, &remote_registry, vec![]);
 
-        remote_registry.shutdown_handle().shutdown();
+        let mut shutdown_handle = remote_registry
+            .take_shutdown_handle()
+            .expect("Unable to get shutdown handle");
+        shutdown_handle.signal_shutdown();
+        shutdown_handle
+            .wait_for_shutdown()
+            .expect("Unable to shutdown remote registry");
         test_config.shutdown();
     }
 
@@ -452,14 +503,20 @@ mod tests {
         registry[0].identity = "".into();
         let test_config = TestConfig::setup("empty_identity", Some(registry));
 
-        let remote_registry =
+        let mut remote_registry =
             RemoteYamlRegistry::new(test_config.url(), test_config.path(), None, None)
                 .expect("Failed to create registry");
 
         // Verify that the registry is still empty
         verify_internal_cache(&test_config, &remote_registry, vec![]);
 
-        remote_registry.shutdown_handle().shutdown();
+        let mut shutdown_handle = remote_registry
+            .take_shutdown_handle()
+            .expect("Unable to get shutdown handle");
+        shutdown_handle.signal_shutdown();
+        shutdown_handle
+            .wait_for_shutdown()
+            .expect("Unable to shutdown remote registry");
         test_config.shutdown();
     }
 
@@ -471,14 +528,20 @@ mod tests {
         registry[0].endpoints = vec!["".into()];
         let test_config = TestConfig::setup("empty_endpoint", Some(registry));
 
-        let remote_registry =
+        let mut remote_registry =
             RemoteYamlRegistry::new(test_config.url(), test_config.path(), None, None)
                 .expect("Failed to create registry");
 
         // Verify that the registry is still empty
         verify_internal_cache(&test_config, &remote_registry, vec![]);
 
-        remote_registry.shutdown_handle().shutdown();
+        let mut shutdown_handle = remote_registry
+            .take_shutdown_handle()
+            .expect("Unable to get shutdown handle");
+        shutdown_handle.signal_shutdown();
+        shutdown_handle
+            .wait_for_shutdown()
+            .expect("Unable to shutdown remote registry");
         test_config.shutdown();
     }
 
@@ -490,14 +553,20 @@ mod tests {
         registry[0].display_name = "".into();
         let test_config = TestConfig::setup("empty_display_name", Some(registry));
 
-        let remote_registry =
+        let mut remote_registry =
             RemoteYamlRegistry::new(test_config.url(), test_config.path(), None, None)
                 .expect("Failed to create registry");
 
         // Verify that the registry is still empty
         verify_internal_cache(&test_config, &remote_registry, vec![]);
 
-        remote_registry.shutdown_handle().shutdown();
+        let mut shutdown_handle = remote_registry
+            .take_shutdown_handle()
+            .expect("Unable to get shutdown handle");
+        shutdown_handle.signal_shutdown();
+        shutdown_handle
+            .wait_for_shutdown()
+            .expect("Unable to shutdown remote registry");
         test_config.shutdown();
     }
 
@@ -509,14 +578,20 @@ mod tests {
         registry[0].keys = vec!["".into()];
         let test_config = TestConfig::setup("empty_key", Some(registry));
 
-        let remote_registry =
+        let mut remote_registry =
             RemoteYamlRegistry::new(test_config.url(), test_config.path(), None, None)
                 .expect("Failed to create registry");
 
         // Verify that the registry is still empty
         verify_internal_cache(&test_config, &remote_registry, vec![]);
 
-        remote_registry.shutdown_handle().shutdown();
+        let mut shutdown_handle = remote_registry
+            .take_shutdown_handle()
+            .expect("Unable to get shutdown handle");
+        shutdown_handle.signal_shutdown();
+        shutdown_handle
+            .wait_for_shutdown()
+            .expect("Unable to shutdown remote registry");
         test_config.shutdown();
     }
 
@@ -527,14 +602,20 @@ mod tests {
         registry[0].endpoints = vec![];
         let test_config = TestConfig::setup("missing_endpoints", Some(registry));
 
-        let remote_registry =
+        let mut remote_registry =
             RemoteYamlRegistry::new(test_config.url(), test_config.path(), None, None)
                 .expect("Failed to create registry");
 
         // Verify that the registry is still empty
         verify_internal_cache(&test_config, &remote_registry, vec![]);
 
-        remote_registry.shutdown_handle().shutdown();
+        let mut shutdown_handle = remote_registry
+            .take_shutdown_handle()
+            .expect("Unable to get shutdown handle");
+        shutdown_handle.signal_shutdown();
+        shutdown_handle
+            .wait_for_shutdown()
+            .expect("Unable to shutdown remote registry");
         test_config.shutdown();
     }
 
@@ -545,14 +626,20 @@ mod tests {
         registry[0].keys = vec![];
         let test_config = TestConfig::setup("missing_keys", Some(registry));
 
-        let remote_registry =
+        let mut remote_registry =
             RemoteYamlRegistry::new(test_config.url(), test_config.path(), None, None)
                 .expect("Failed to create registry");
 
         // Verify that the registry is still empty
         verify_internal_cache(&test_config, &remote_registry, vec![]);
 
-        remote_registry.shutdown_handle().shutdown();
+        let mut shutdown_handle = remote_registry
+            .take_shutdown_handle()
+            .expect("Unable to get shutdown handle");
+        shutdown_handle.signal_shutdown();
+        shutdown_handle
+            .wait_for_shutdown()
+            .expect("Unable to shutdown remote registry");
         test_config.shutdown();
     }
 
@@ -561,18 +648,24 @@ mod tests {
     fn fetch_node_ok() {
         let test_config = TestConfig::setup("fetch_node_ok", Some(mock_registry()));
 
-        let remote_registry =
+        let mut remote_registry =
             RemoteYamlRegistry::new(test_config.url(), test_config.path(), None, None)
                 .expect("Failed to create registry");
 
         let expected_node = mock_registry().pop().expect("Failed to get expected node");
         let node = remote_registry
-            .fetch_node(&expected_node.identity)
+            .get_node(&expected_node.identity)
             .expect("Failed to fetch node")
             .expect("Node not found");
         assert_eq!(node, expected_node);
 
-        remote_registry.shutdown_handle().shutdown();
+        let mut shutdown_handle = remote_registry
+            .take_shutdown_handle()
+            .expect("Unable to get shutdown handle");
+        shutdown_handle.signal_shutdown();
+        shutdown_handle
+            .wait_for_shutdown()
+            .expect("Unable to shutdown remote registry");
         test_config.shutdown();
     }
 
@@ -581,16 +674,51 @@ mod tests {
     fn fetch_node_not_found() {
         let test_config = TestConfig::setup("fetch_node_not_found", Some(mock_registry()));
 
-        let remote_registry =
+        let mut remote_registry =
             RemoteYamlRegistry::new(test_config.url(), test_config.path(), None, None)
                 .expect("Failed to create registry");
 
         assert!(remote_registry
-            .fetch_node("NodeNotInRegistry")
+            .get_node("NodeNotInRegistry")
             .expect("Failed to fetch node")
             .is_none());
 
-        remote_registry.shutdown_handle().shutdown();
+        let mut shutdown_handle = remote_registry
+            .take_shutdown_handle()
+            .expect("Unable to get shutdown handle");
+        shutdown_handle.signal_shutdown();
+        shutdown_handle
+            .wait_for_shutdown()
+            .expect("Unable to shutdown remote registry");
+        test_config.shutdown();
+    }
+
+    ///
+    /// Verifies that `has_node` properly determines if a node exists in the registry.
+    ///
+    #[test]
+    fn has_node() {
+        let test_config = TestConfig::setup("has_node", Some(mock_registry()));
+
+        let mut remote_registry =
+            RemoteYamlRegistry::new(test_config.url(), test_config.path(), None, None)
+                .expect("Failed to create registry");
+
+        let expected_node = mock_registry().pop().expect("Failed to get expected node");
+        assert!(remote_registry
+            .has_node(&expected_node.identity)
+            .expect("Failed to check if expected_node exists"));
+        assert!(!remote_registry
+            .has_node("NodeNotInRegistry")
+            .expect("Failed to check for non-existent node"));
+
+        let mut shutdown_handle = remote_registry
+            .take_shutdown_handle()
+            .expect("Unable to get shutdown handle");
+        shutdown_handle.signal_shutdown();
+        shutdown_handle
+            .wait_for_shutdown()
+            .expect("Unable to shutdown remote registry");
         test_config.shutdown();
     }
 
@@ -599,7 +727,7 @@ mod tests {
     fn list_nodes() {
         let test_config = TestConfig::setup("list_nodes", Some(mock_registry()));
 
-        let remote_registry =
+        let mut remote_registry =
             RemoteYamlRegistry::new(test_config.url(), test_config.path(), None, None)
                 .expect("Failed to create registry");
 
@@ -613,7 +741,13 @@ mod tests {
             assert!(nodes.contains(&node));
         }
 
-        remote_registry.shutdown_handle().shutdown();
+        let mut shutdown_handle = remote_registry
+            .take_shutdown_handle()
+            .expect("Unable to get shutdown handle");
+        shutdown_handle.signal_shutdown();
+        shutdown_handle
+            .wait_for_shutdown()
+            .expect("Unable to shutdown remote registry");
         test_config.shutdown();
     }
 
@@ -622,7 +756,7 @@ mod tests {
     fn list_nodes_empty() {
         let test_config = TestConfig::setup("list_nodes_empty", Some(vec![]));
 
-        let remote_registry =
+        let mut remote_registry =
             RemoteYamlRegistry::new(test_config.url(), test_config.path(), None, None)
                 .expect("Failed to create registry");
 
@@ -633,7 +767,13 @@ mod tests {
 
         assert!(nodes.is_empty());
 
-        remote_registry.shutdown_handle().shutdown();
+        let mut shutdown_handle = remote_registry
+            .take_shutdown_handle()
+            .expect("Unable to get shutdown handle");
+        shutdown_handle.signal_shutdown();
+        shutdown_handle
+            .wait_for_shutdown()
+            .expect("Unable to shutdown remote registry");
         test_config.shutdown();
     }
 
@@ -642,7 +782,7 @@ mod tests {
     fn list_nodes_filter_metadata() {
         let test_config = TestConfig::setup("list_nodes_filter_metadata", Some(mock_registry()));
 
-        let remote_registry =
+        let mut remote_registry =
             RemoteYamlRegistry::new(test_config.url(), test_config.path(), None, None)
                 .expect("Failed to create registry");
 
@@ -663,7 +803,13 @@ mod tests {
         assert_eq!(nodes.len(), 1);
         assert_eq!(nodes[0], mock_registry()[0]);
 
-        remote_registry.shutdown_handle().shutdown();
+        let mut shutdown_handle = remote_registry
+            .take_shutdown_handle()
+            .expect("Unable to get shutdown handle");
+        shutdown_handle.signal_shutdown();
+        shutdown_handle
+            .wait_for_shutdown()
+            .expect("Unable to shutdown remote registry");
         test_config.shutdown();
     }
 
@@ -673,7 +819,7 @@ mod tests {
     fn list_nodes_filter_multiple() {
         let test_config = TestConfig::setup("list_nodes_filter_multiple", Some(mock_registry()));
 
-        let remote_registry =
+        let mut remote_registry =
             RemoteYamlRegistry::new(test_config.url(), test_config.path(), None, None)
                 .expect("Failed to create registry");
 
@@ -704,7 +850,13 @@ mod tests {
         assert_eq!(nodes.len(), 1);
         assert_eq!(nodes[0], mock_registry()[2]);
 
-        remote_registry.shutdown_handle().shutdown();
+        let mut shutdown_handle = remote_registry
+            .take_shutdown_handle()
+            .expect("Unable to get shutdown handle");
+        shutdown_handle.signal_shutdown();
+        shutdown_handle
+            .wait_for_shutdown()
+            .expect("Unable to shutdown remote registry");
         test_config.shutdown();
     }
 
@@ -713,7 +865,7 @@ mod tests {
     fn list_nodes_filter_empty() {
         let test_config = TestConfig::setup("list_nodes_filter_empty", Some(mock_registry()));
 
-        let remote_registry =
+        let mut remote_registry =
             RemoteYamlRegistry::new(test_config.url(), test_config.path(), None, None)
                 .expect("Failed to create registry");
 
@@ -729,7 +881,13 @@ mod tests {
 
         assert!(nodes.is_empty());
 
-        remote_registry.shutdown_handle().shutdown();
+        let mut shutdown_handle = remote_registry
+            .take_shutdown_handle()
+            .expect("Unable to get shutdown handle");
+        shutdown_handle.signal_shutdown();
+        shutdown_handle
+            .wait_for_shutdown()
+            .expect("Unable to shutdown remote registry");
         test_config.shutdown();
     }
 
@@ -739,13 +897,19 @@ mod tests {
     fn file_available_at_startup() {
         let test_config = TestConfig::setup("file_available_at_startup", Some(mock_registry()));
 
-        let remote_registry =
+        let mut remote_registry =
             RemoteYamlRegistry::new(test_config.url(), test_config.path(), None, None)
                 .expect("Failed to create registry");
 
         verify_internal_cache(&test_config, &remote_registry, mock_registry());
 
-        remote_registry.shutdown_handle().shutdown();
+        let mut shutdown_handle = remote_registry
+            .take_shutdown_handle()
+            .expect("Unable to get shutdown handle");
+        shutdown_handle.signal_shutdown();
+        shutdown_handle
+            .wait_for_shutdown()
+            .expect("Unable to shutdown remote registry");
         test_config.shutdown();
     }
 
@@ -757,7 +921,7 @@ mod tests {
         // Start without a remote file
         let test_config = TestConfig::setup("file_unavailable_at_startup", None);
 
-        let remote_registry =
+        let mut remote_registry =
             RemoteYamlRegistry::new(test_config.url(), test_config.path(), None, None)
                 .expect("Failed to create registry");
 
@@ -770,7 +934,13 @@ mod tests {
         // Verify that the registry's contents were updated
         verify_internal_cache(&test_config, &remote_registry, mock_registry());
 
-        remote_registry.shutdown_handle().shutdown();
+        let mut shutdown_handle = remote_registry
+            .take_shutdown_handle()
+            .expect("Unable to get shutdown handle");
+        shutdown_handle.signal_shutdown();
+        shutdown_handle
+            .wait_for_shutdown()
+            .expect("Unable to shutdown remote registry");
         test_config.shutdown();
     }
 
@@ -779,14 +949,21 @@ mod tests {
     fn auto_refresh_disabled() {
         let test_config = TestConfig::setup("auto_refresh_disabled", Some(mock_registry()));
 
-        let remote_registry =
+        let mut remote_registry =
             RemoteYamlRegistry::new(test_config.url(), test_config.path(), None, None)
                 .expect("Failed to create registry");
 
-        // The `running` atomic bool is only set if the auto refresh thread was started.
-        assert!(remote_registry.shutdown_handle().running.is_none());
+        let mut shutdown_handle = remote_registry
+            .take_shutdown_handle()
+            .expect("Unable to get shutdown handle");
 
-        remote_registry.shutdown_handle().shutdown();
+        // The `running` atomic bool is only set if the auto refresh thread was started.
+        assert!(shutdown_handle.running.is_none());
+
+        shutdown_handle.signal_shutdown();
+        shutdown_handle
+            .wait_for_shutdown()
+            .expect("Unable to shutdown remote registry");
         test_config.shutdown();
     }
 
@@ -797,7 +974,7 @@ mod tests {
         let test_config = TestConfig::setup("auto_refresh_enabled", Some(mock_registry()));
 
         let refresh_period = Duration::from_secs(1);
-        let remote_registry = RemoteYamlRegistry::new(
+        let mut remote_registry = RemoteYamlRegistry::new(
             test_config.url(),
             test_config.path(),
             Some(refresh_period),
@@ -807,8 +984,12 @@ mod tests {
 
         verify_internal_cache(&test_config, &remote_registry, mock_registry());
 
+        let mut shutdown_handle = remote_registry
+            .take_shutdown_handle()
+            .expect("Unable to get shutdown handle");
+
         // The `running` atomic bool is only set if the auto refresh thread was started.
-        assert!(remote_registry.shutdown_handle().running.is_some());
+        assert!(shutdown_handle.running.is_some());
 
         test_config.update_registry(Some(vec![]));
 
@@ -818,7 +999,10 @@ mod tests {
         // Verify that the registry's contents were updated
         verify_internal_cache(&test_config, &remote_registry, vec![]);
 
-        remote_registry.shutdown_handle().shutdown();
+        shutdown_handle.signal_shutdown();
+        shutdown_handle
+            .wait_for_shutdown()
+            .expect("Unable to shutdown remote registry");
         test_config.shutdown();
     }
 
@@ -828,7 +1012,7 @@ mod tests {
     fn forced_refresh_disabled() {
         let test_config = TestConfig::setup("forced_refresh_disabled", Some(mock_registry()));
 
-        let remote_registry =
+        let mut remote_registry =
             RemoteYamlRegistry::new(test_config.url(), test_config.path(), None, None)
                 .expect("Failed to create registry");
 
@@ -840,7 +1024,13 @@ mod tests {
         // was updated
         verify_internal_cache(&test_config, &remote_registry, mock_registry());
 
-        remote_registry.shutdown_handle().shutdown();
+        let mut shutdown_handle = remote_registry
+            .take_shutdown_handle()
+            .expect("Unable to get shutdown handle");
+        shutdown_handle.signal_shutdown();
+        shutdown_handle
+            .wait_for_shutdown()
+            .expect("Unable to shutdown remote registry");
         test_config.shutdown();
     }
 
@@ -851,7 +1041,7 @@ mod tests {
         let test_config = TestConfig::setup("forced_refresh_enabled", Some(mock_registry()));
 
         let refresh_period = Duration::from_millis(10);
-        let remote_registry = RemoteYamlRegistry::new(
+        let mut remote_registry = RemoteYamlRegistry::new(
             test_config.url(),
             test_config.path(),
             None,
@@ -869,7 +1059,13 @@ mod tests {
         // Verify that the registry's contents are updated on read
         verify_internal_cache(&test_config, &remote_registry, vec![]);
 
-        remote_registry.shutdown_handle().shutdown();
+        let mut shutdown_handle = remote_registry
+            .take_shutdown_handle()
+            .expect("Unable to get shutdown handle");
+        shutdown_handle.signal_shutdown();
+        shutdown_handle
+            .wait_for_shutdown()
+            .expect("Unable to shutdown remote registry");
         test_config.shutdown();
     }
 
@@ -880,22 +1076,35 @@ mod tests {
         let test_config = TestConfig::setup("restart_file_available", Some(mock_registry()));
 
         // Start the registry the first time, verify its contents, and shut it down
-        let remote_registry =
+        let mut remote_registry =
             RemoteYamlRegistry::new(test_config.url(), test_config.path(), None, None)
                 .expect("Failed to create registry");
         verify_internal_cache(&test_config, &remote_registry, mock_registry());
-        remote_registry.shutdown_handle().shutdown();
+
+        let mut shutdown_handle = remote_registry
+            .take_shutdown_handle()
+            .expect("Unable to get shutdown handle");
+        shutdown_handle.signal_shutdown();
+        shutdown_handle
+            .wait_for_shutdown()
+            .expect("Unable to shutdown remote registry");
 
         // Update the remote file
         test_config.update_registry(Some(vec![]));
 
         // Start the registry again and verify that it has the updated registry contents
-        let remote_registry =
+        let mut remote_registry =
             RemoteYamlRegistry::new(test_config.url(), test_config.path(), None, None)
                 .expect("Failed to create registry");
         verify_internal_cache(&test_config, &remote_registry, vec![]);
 
-        remote_registry.shutdown_handle().shutdown();
+        let mut shutdown_handle = remote_registry
+            .take_shutdown_handle()
+            .expect("Unable to get shutdown handle");
+        shutdown_handle.signal_shutdown();
+        shutdown_handle
+            .wait_for_shutdown()
+            .expect("Unable to shutdown remote registry");
         test_config.shutdown();
     }
 
@@ -906,22 +1115,34 @@ mod tests {
         let test_config = TestConfig::setup("restart_file_unavailable", Some(mock_registry()));
 
         // Start the registry the first time, verify its contents, and shut it down
-        let remote_registry =
+        let mut remote_registry =
             RemoteYamlRegistry::new(test_config.url(), test_config.path(), None, None)
                 .expect("Failed to create registry");
         verify_internal_cache(&test_config, &remote_registry, mock_registry());
-        remote_registry.shutdown_handle().shutdown();
+        let mut shutdown_handle = remote_registry
+            .take_shutdown_handle()
+            .expect("Unable to get shutdown handle");
+        shutdown_handle.signal_shutdown();
+        shutdown_handle
+            .wait_for_shutdown()
+            .expect("Unable to shutdown remote registry");
 
         // Make the remote file unavailable
         test_config.update_registry(None);
 
         // Start the registry again and verify that the old contents are still available
-        let remote_registry =
+        let mut remote_registry =
             RemoteYamlRegistry::new(test_config.url(), test_config.path(), None, None)
                 .expect("Failed to create registry");
         verify_internal_cache(&test_config, &remote_registry, mock_registry());
 
-        remote_registry.shutdown_handle().shutdown();
+        let mut shutdown_handle = remote_registry
+            .take_shutdown_handle()
+            .expect("Unable to get shutdown handle");
+        shutdown_handle.signal_shutdown();
+        shutdown_handle
+            .wait_for_shutdown()
+            .expect("Unable to shutdown remote registry");
         test_config.shutdown();
     }
 
@@ -974,9 +1195,14 @@ mod tests {
         let filename = compute_cache_filename(test_config.url(), test_config.path())
             .expect("Failed to compute cache filename");
         let file = File::open(filename).expect("Failed to open cache file");
-        let file_contents: Vec<Node> =
+        let file_contents: Vec<YamlNode> =
             serde_yaml::from_reader(file).expect("Failed to deserialize cache file");
-        assert_eq!(file_contents, expected_registry);
+
+        let file_contents_nodes: Vec<Node> = file_contents
+            .into_iter()
+            .map(|node| Node::try_from(node).expect("Unable to build node"))
+            .collect();
+        assert_eq!(file_contents_nodes, expected_registry);
     }
 
     /// Simplifies tests by handling some of the setup and tear down.
@@ -1050,43 +1276,76 @@ mod tests {
     fn serve_registry(
         registry: Arc<Mutex<Option<Vec<Node>>>>,
     ) -> (RestApiShutdownHandle, std::thread::JoinHandle<()>, String) {
-        let (shutdown, join, url) = run_rest_api_on_open_port(vec![Resource::build(
-            "/registry.yaml",
-        )
-        .add_method(Method::Get, move |_, _| {
-            Box::new(match &*registry.lock().expect("Registry lock poisoned") {
-                Some(registry) => HttpResponse::Ok()
-                    .body(serde_yaml::to_vec(&registry).expect("Failed to serialize registry file"))
-                    .into_future(),
-                None => HttpResponse::NotFound().finish().into_future(),
+        let mut resource = Resource::build("/registry.yaml");
+        #[cfg(feature = "authorization")]
+        {
+            resource = resource.add_method(
+                Method::Get,
+                Permission::AllowUnauthenticated,
+                move |_, _| {
+                    Box::new(match &*registry.lock().expect("Registry lock poisoned") {
+                        Some(registry) => {
+                            let yaml_registry: Vec<YamlNode> = registry
+                                .iter()
+                                .map(|node| YamlNode::from(node.clone()))
+                                .collect();
+                            HttpResponse::Ok()
+                                .body(
+                                    serde_yaml::to_vec(&yaml_registry)
+                                        .expect("Failed to serialize registry file"),
+                                )
+                                .into_future()
+                        }
+                        None => HttpResponse::NotFound().finish().into_future(),
+                    })
+                },
+            )
+        }
+        #[cfg(not(feature = "authorization"))]
+        {
+            resource = resource.add_method(Method::Get, move |_, _| {
+                Box::new(match &*registry.lock().expect("Registry lock poisoned") {
+                    Some(registry) => {
+                        let yaml_registry: Vec<YamlNode> = registry
+                            .iter()
+                            .map(|node| YamlNode::from(node.clone()))
+                            .collect();
+                        HttpResponse::Ok()
+                            .body(
+                                serde_yaml::to_vec(&yaml_registry)
+                                    .expect("Failed to serialize registry file"),
+                            )
+                            .into_future()
+                    }
+                    None => HttpResponse::NotFound().finish().into_future(),
+                })
             })
-        })]);
+        }
+        let (shutdown, join, url) = run_rest_api_on_open_port(vec![resource]);
 
         (shutdown, join, format!("http://{}/registry.yaml", url))
     }
 
-    /// Runs a REST API with the given `resources` on an open port. Returned string is the URL the
-    /// REST API is bound to.
     fn run_rest_api_on_open_port(
         resources: Vec<Resource>,
     ) -> (RestApiShutdownHandle, std::thread::JoinHandle<()>, String) {
-        (10000..20000)
-            .find_map(|port| {
-                let bind_url = format!("127.0.0.1:{}", port);
-                let result = RestApiBuilder::new()
-                    .with_bind(&bind_url)
-                    .add_resources(resources.clone())
-                    .build()
-                    .expect("Failed to build REST API")
-                    .run();
-                match result {
-                    Ok((shutdown_handle, join_handle)) => {
-                        Some((shutdown_handle, join_handle, bind_url))
-                    }
-                    Err(RestApiServerError::BindError(_)) => None,
-                    Err(err) => panic!("Failed to run REST API: {}", err),
-                }
-            })
-            .expect("No port available")
+        #[cfg(not(feature = "https-bind"))]
+        let bind = "127.0.0.1:0";
+        #[cfg(feature = "https-bind")]
+        let bind = crate::rest_api::BindConfig::Http("127.0.0.1:0".into());
+
+        let result = RestApiBuilder::new()
+            .with_bind(bind)
+            .add_resources(resources.clone())
+            .build_insecure()
+            .expect("Failed to build REST API")
+            .run_insecure();
+        match result {
+            Ok((shutdown_handle, join_handle)) => {
+                let port = shutdown_handle.port_numbers()[0];
+                (shutdown_handle, join_handle, format!("127.0.0.1:{}", port))
+            }
+            Err(err) => panic!("Failed to run REST API: {}", err),
+        }
     }
 }
